package cd.casic.ci.process.process.service.stage.impl;

import cd.casic.ci.common.pipeline.constant.PipelineDateUtilConstant;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.req.stage.StageReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;
import cd.casic.ci.common.pipeline.utils.PipelineDateUtil;
import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.util.object.BeanUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.task.VirtualThreadTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class StageServiceImpl implements StageService {
    @Resource
    private TaskService taskService;
    @Resource
    private PipStageDao stageDao;

    @Override
    public String createStagesOrTask(StageReq stageReq) {
        PipStage stage = new PipStage();
        BeanUtils.copyProperties(stageReq,stage);
        String stagesId = stage.getStageId();
        String pipelineId = stage.getPipelineId();

        String taskType = stage.getTaskType();
        int stageSort = stage.getStageSort();

        stage.setCreateTime(PipelineDateUtil.nowDateString(PipelineDateUtilConstant.yyyy_MM_dd_HH_mm_ss));

        PipTask tasks = new PipTask();
        tasks.setTaskType(taskType);
        tasks.setTaskSort(1);
        // TODO 查询是否为已知worker
        taskService.taskTypeExist(taskType);
        //是否为源码
        if (taskType.equals(PipelineFinal.TASK_TYPE_CODE)){
            //判断是否存在代码源(目标)节点
            findTargetTasks(pipelineId);

            //创建根节点
            int initStage = initStage(pipelineId,1);
            stage.setCode(true);
            stage.setStageSort(1);
            stage.setPipelineId(pipelineId);
            stage.setStageName("阶段-"+initStage);
            String id = createStages(stage);

            //从节点
            PipStage stages = new PipStage();
            stages.setStageSort(1);
            stages.setParentId(id);
            stages.setStageName("源码");
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);

            return taskService.createTasksOrTask(tasks);
        }

        //新任务
        if (!StringUtils.isNotEmpty(stagesId) && stageSort != 0){
            //创建主节点
            int initStage = initStage(pipelineId, stageSort);
            stage.setStageSort(initStage);
            stage.setPipelineId(pipelineId);
            stage.setStageName("阶段-" + initStage);
            String id = createStages(stage);

            //创建从节点
            PipStage stages = new PipStage();
            stages.setParentId(id);
            stages.setStageSort(1);
            stages.setStageName("并行阶段-"+initStage+"-" + 1);
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);
            return taskService.createTasksOrTask(tasks);
        }

        //并行任务
        if (StringUtils.isNotEmpty(stagesId)&& stageSort == 0){
            List<PipStage> otherStage = findOtherStage(stagesId);
            PipStage oneStages = findOneStages(stagesId);
            PipStage stages = new PipStage();
            stages.setParentId(stagesId);
            stages.setStageSort(otherStage.size() + 1);
            stages.setStageName("并行阶段-"+oneStages.getStageSort()+"-"+(otherStage.size()+1));
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);
            return taskService.createTasksOrTask(tasks);
        }

        //串行任务
        int sort = stage.getTaskSort();
        if (sort != 0){
            tasks.setTaskSort(sort);
            tasks.setStageId(stagesId);
            return tasksService.createTasksOrTask(tasks);
        }
        throw new ApplicationException(50001,"未知的操作类型");
    }
    /**
     * 更新主节点阶段顺序
     * @param pipelineId 流水线id
     * @param taskSort 顺序
     * @return 顺序
     */
    private Integer initStage(String pipelineId,int taskSort){
        List<PipStage> allMainStage = findAllMainStage(pipelineId);
        if (allMainStage.isEmpty()){
            return 1;
        }
        for (PipStage stages : allMainStage) {
            int stage = stages.getStageSort();
            if (stage < taskSort){
                continue;
            }
            stages.setStageSort(stage+1);
            updateStages(stages);
        }
        return taskSort;
    }
    @Override
    public void createStageTemplate(String pipelineId, String[] template) {

    }

    @Override
    public void cloneStage(String pipelineId, String clonePipelineId) {

    }

    @Override
    public List<StageResp> findAllStagesOrTask(String pipelineId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesTask(String pipelineId) {
        return null;
    }

    @Override
    public void deleteStagesOrTask(String taskId) {

    }

    @Override
    public void deleteAllStagesOrTask(String pipelineId) {

    }

    @Override
    public void updateStageName(StageReq stage) {

    }

    @Override
    public List<PipStage> findAllMainStage(String pipelineId) {
        return null;
    }

    @Override
    public List<PipStage> findOtherStage(String stagesId) {
        return null;
    }

    @Override
    public void updateStagesTask(StageReq stage) {

    }

    @Override
    public List<String> validStagesMustField(String pipelineId) {
        return null;
    }

    @Override
    public String createStages(PipStage stage) {
        return null;
    }

    @Override
    public void updateStages(PipStage stage) {
        stageDao.updateById(stage);
    }

    @Override
    public void deleteStages(String stageId) {

    }

    @Override
    public StageResp findOneStages(String stageId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStages() {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesList(List<String> idList) {
        return null;
    }
    /**
     * 判断任务是否存在代码源
     * @param pipelineId 流水线id
     */
    private void findTargetTasks(String pipelineId) {
        List<StageResp> allStage = findAllMainStage(pipelineId);
        if (allStage.isEmpty()){
            return;
        }
        for (StageResp stage : allStage) {
            if (stage.isCode()){
                throw new ServiceException(50001,"代码源已存在，无法再次创建。");
            }
        }
    }
}
